from .exceptions import JenkinsError as JenkinsError
from typing import Any, List, Tuple

JOB_BUILD_URL_RE: Any

def construct_job_config(*, description: str=..., parameters: List[dict]=..., commands: List[str]=...) -> str: ...
def construct_node_config(name: str, *, remote_fs: str=..., executors: int=...) -> dict: ...
def parse_build_url(build_url: str) -> Tuple[str, int]: ...
